// -*- mode: C++ -*-

// AUTOGENERATED BY glean_parser.  DO NOT EDIT.

/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#ifndef mozilla_Metrics_h
#define mozilla_Metrics_h

#include "mozilla/glean/bindings/MetricTypes.h"
#include "mozilla/Tuple.h"
#include "mozilla/Maybe.h"
#include "nsTArray.h"
#include "nsPrintfCString.h"

namespace mozilla::glean {
struct NoExtraKeys;

namespace newtab {
    /**
     * generated from newtab.locale
     */
    /**
     * The application's locale as of when newtab's TelemetryFeed was init. Comes from
     * `Services.local.appLocaleAsBCP47`. Looks like `en-US`.
     */
    constexpr impl::StringMetric locale(1);

    /**
     * generated from newtab.opened
     */
    struct OpenedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> source;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (source) {
          extraKeys.AppendElement()->AssignASCII("source");
          extraValues.EmplaceBack(source.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when newtab UI is opened via `about:newtab` or `about:home` or
     * `about:welcome` and has been made visible (see `visibility_event_rcvd_ts` in
     * [detect-user-session-start.js](https://searchfox.org/mozilla-
     * central/source/browser/components/newtab/content-src/lib/detect-user-session-
     * start.js)).
     */
    constexpr impl::EventMetric<OpenedExtra> opened(2);

    /**
     * generated from newtab.closed
     */
    struct ClosedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when newtab UI is closed by * navigation * closing the tab
     * Doesn't mean that the newtab was ever visible to a user.
     */
    constexpr impl::EventMetric<ClosedExtra> closed(3);

}
namespace topsites {
    /**
     * generated from topsites.impression
     */
    struct ImpressionExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when topsite tiles are loaded. Presently only happens for sponsored
     * tiles.
     */
    constexpr impl::EventMetric<ImpressionExtra> impression(4);

    /**
     * generated from topsites.click
     */
    struct ClickExtra {
      mozilla::Maybe<bool> isSponsored;
      mozilla::Maybe<nsCString> newtabVisitId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isSponsored) {
          extraKeys.AppendElement()->AssignASCII("is_sponsored");
          extraValues.AppendElement()->AssignASCII(isSponsored.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a topsite tile is clicked. Presently only happens for sponsored
     * tiles.
     */
    constexpr impl::EventMetric<ClickExtra> click(5);

}
namespace newtab_search {
    /**
     * generated from newtab.search.issued
     */
    struct IssuedExtra {
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * When Firefox was asked to issue a search from a Search Access Point (SAP) on a
     * newtab page. Doesn't record searches in Private Browsing Mode unless
     * `browser.engagement.search_counts.pbm` is set to `true`.
     */
    constexpr impl::EventMetric<IssuedExtra> issued(6);

}
namespace newtab_search_ad {
    /**
     * generated from newtab.search.ad.impression
     */
    struct ImpressionExtra {
      mozilla::Maybe<bool> isFollowOn;
      mozilla::Maybe<bool> isTagged;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isFollowOn) {
          extraKeys.AppendElement()->AssignASCII("is_follow_on");
          extraValues.AppendElement()->AssignASCII(isFollowOn.value() ? "true" : "false");
        }
        if (isTagged) {
          extraKeys.AppendElement()->AssignASCII("is_tagged");
          extraValues.AppendElement()->AssignASCII(isTagged.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a newtab visit resulted in a search that loaded a Search Engine
     * Result Page (SERP) that contains an ad link. And the SERP is visible.
     */
    constexpr impl::EventMetric<ImpressionExtra> impression(7);

    /**
     * generated from newtab.search.ad.click
     */
    struct ClickExtra {
      mozilla::Maybe<bool> isFollowOn;
      mozilla::Maybe<bool> isTagged;
      mozilla::Maybe<nsCString> newtabVisitId;
      mozilla::Maybe<nsCString> searchAccessPoint;
      mozilla::Maybe<nsCString> telemetryId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (isFollowOn) {
          extraKeys.AppendElement()->AssignASCII("is_follow_on");
          extraValues.AppendElement()->AssignASCII(isFollowOn.value() ? "true" : "false");
        }
        if (isTagged) {
          extraKeys.AppendElement()->AssignASCII("is_tagged");
          extraValues.AppendElement()->AssignASCII(isTagged.value() ? "true" : "false");
        }
        if (newtabVisitId) {
          extraKeys.AppendElement()->AssignASCII("newtab_visit_id");
          extraValues.EmplaceBack(newtabVisitId.value());
        }
        if (searchAccessPoint) {
          extraKeys.AppendElement()->AssignASCII("search_access_point");
          extraValues.EmplaceBack(searchAccessPoint.value());
        }
        if (telemetryId) {
          extraKeys.AppendElement()->AssignASCII("telemetry_id");
          extraValues.EmplaceBack(telemetryId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an ad link is clicked on a Search Engine Result Page (SERP) which
     * was loaded by a seach that began on a newtab page.
     */
    constexpr impl::EventMetric<ClickExtra> click(8);

}
namespace browser_engagement {
    /**
     * generated from browser.engagement.active_ticks
     */
    /**
     * The number of five-second intervals ('ticks') the user was considered
     * 'active'.
     * 
     * 'active' means keyboard or mouse interaction with the application.
     * It doesn't take into account whether or not the window has focus or is in
     * the foreground, only if it is receiving these interaction events.
     * 
     * Migrated from Telemetry's `browser.engagement.active_ticks`.
     */
    constexpr impl::CounterMetric active_ticks(9);

    /**
     * generated from browser.engagement.uri_count
     */
    /**
     * The number of total non-unique http(s) URIs visited, including page
     * reloads, after the session has been restored. URIs on minimized or
     * background tabs may also be counted. Private browsing uris are included.
     * 
     * Migrated from Telemetry's
     * `browser.engagement.total_uri_count_normal_and_private_mode`.
     */
    constexpr impl::CounterMetric uri_count(10);

}
namespace gmp {
    /**
     * generated from gmp.update_xml_fetch_result
     */
    /**
     * The result of Gecko fetching an update.xml from Balrog. This captures 3
     * different data points: success or failure of the request, if cert pinning or
     * content signatures were used to verify the result, and the reason for failure,
     * if the request failed.
     */
    constexpr impl::Labeled<impl::CounterMetric> update_xml_fetch_result(11);

}
namespace perf {
    /**
     * generated from perf.page_load
     */
    struct PageLoadExtra {
      mozilla::Maybe<uint32_t> fcpTime;
      mozilla::Maybe<uint32_t> jsExecTime;
      mozilla::Maybe<uint32_t> loadTime;
      mozilla::Maybe<nsCString> loadType;
      mozilla::Maybe<uint32_t> responseTime;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (fcpTime) {
          extraKeys.AppendElement()->AssignASCII("fcp_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", fcpTime.value()));
        }
        if (jsExecTime) {
          extraKeys.AppendElement()->AssignASCII("js_exec_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", jsExecTime.value()));
        }
        if (loadTime) {
          extraKeys.AppendElement()->AssignASCII("load_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", loadTime.value()));
        }
        if (loadType) {
          extraKeys.AppendElement()->AssignASCII("load_type");
          extraValues.EmplaceBack(loadType.value());
        }
        if (responseTime) {
          extraKeys.AppendElement()->AssignASCII("response_time");
          extraValues.EmplaceBack(nsPrintfCString("%d", responseTime.value()));
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a top level content document has been loaded.
     */
    constexpr impl::EventMetric<PageLoadExtra> page_load(12);

}
namespace paint {
    /**
     * generated from paint.build_displaylist_time
     */
    /**
     * The time to build a Gecko display list.
     */
    constexpr impl::TimingDistributionMetric build_displaylist_time(13);

}
namespace wr {
    /**
     * generated from wr.gpu_wait_time
     */
    /**
     * The time spent waiting for the GPU to complete previously issued drawing
     * commands.
     */
    constexpr impl::TimingDistributionMetric gpu_wait_time(14);

    /**
     * generated from wr.rasterize_glyphs_time
     */
    /**
     * The time to rasterize glyphs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric rasterize_glyphs_time(15);

    /**
     * generated from wr.rasterize_blobs_time
     */
    /**
     * The time to rasterize blobs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric rasterize_blobs_time(16);

    /**
     * generated from wr.renderer_time
     */
    /**
     * WebRender renderer time.
     */
    constexpr impl::TimingDistributionMetric renderer_time(17);

    /**
     * generated from wr.framebuild_time
     */
    /**
     * The time to rasterize glyphs for consumption by WebRender.
     */
    constexpr impl::TimingDistributionMetric framebuild_time(18);

    /**
     * generated from wr.scenebuild_time
     */
    /**
     * WebRender scene build time.
     */
    constexpr impl::TimingDistributionMetric scenebuild_time(19);

    /**
     * generated from wr.sceneswap_time
     */
    /**
     * WebRender scene swap time.
     */
    constexpr impl::TimingDistributionMetric sceneswap_time(20);

    /**
     * generated from wr.texture_cache_update_time
     */
    /**
     * Time taken by WebRender to update the texture cache.
     */
    constexpr impl::TimingDistributionMetric texture_cache_update_time(21);

    /**
     * generated from wr.time_to_frame_build
     */
    /**
     * Time elapsed between the construction of a transaction and the associated frame
     * build beginning.
     */
    constexpr impl::TimingDistributionMetric time_to_frame_build(22);

    /**
     * generated from wr.time_to_render_start
     */
    /**
     * Time elapsed between the construction of a frame and the start of rendering.
     */
    constexpr impl::TimingDistributionMetric time_to_render_start(23);

}
namespace netwerk {
    /**
     * generated from netwerk.early_hints
     */
    /**
     * Counts the different type of resources that are sent for early hints.
     */
    constexpr impl::Labeled<impl::CounterMetric> early_hints(24);

}
namespace extensions {
    /**
     * generated from extensions.startup_cache_load_time
     */
    /**
     * Time to load and deserialize the extensions startupCache data.
     */
    constexpr impl::TimespanMetric startup_cache_load_time(25);

}
namespace fog {
    /**
     * generated from fog.initialization
     */
    /**
     * Time the FOG initialization takes.
     */
    constexpr impl::TimespanMetric initialization(26);

    /**
     * generated from fog.failed_idle_registration
     */
    /**
     * True if we failed to register with the idle service. Absent otherwise.
     * Means IPC probably isn't working well.
     * Child-process data will likely be absent, or incomplete.
     */
    constexpr impl::BooleanMetric failed_idle_registration(27);

}
namespace fog_ipc {
    /**
     * generated from fog.ipc.replay_failures
     */
    /**
     * The number of times the ipc buffer failed to be replayed in the
     * parent process.
     */
    constexpr impl::CounterMetric replay_failures(28);

    /**
     * generated from fog.ipc.buffer_sizes
     */
    /**
     * The number and size of the IPC buffers being received over FOG IPC.
     */
    constexpr impl::MemoryDistributionMetric buffer_sizes(29);

    /**
     * generated from fog.ipc.flush_durations
     */
    /**
     * The length of time between asking the child processes for their
     * IPC buffers and all of them being received by the parent.
     */
    constexpr impl::TimingDistributionMetric flush_durations(30);

    /**
     * generated from fog.ipc.flush_failures
     */
    /**
     * The number of times we failed to flush all non-parent-process data,
     * throwing even partial results into the trash.
     * If this number is high, we might consider writing custom `MozPromise`-
     * handling code instead of using `MozPromise::All`.
     */
    constexpr impl::CounterMetric flush_failures(31);

    /**
     * generated from fog.ipc.shutdown_registration_failures
     */
    /**
     * The number of times we tried to register shutdown flush routines for
     * content child processes, and failed (probably because there was no main
     * thread).
     * As a result there may be data loss from content child processes.
     * Large or rising number of clients experiencing this indicates we should
     * perhaps refactor content child shutdown in FOG to try harder to register
     * flush operations.
     * Will likely be obsoleted by bug 1641989.
     */
    constexpr impl::CounterMetric shutdown_registration_failures(32);

}
namespace test_only {
    /**
     * generated from test_only.bad_code
     */
    /**
     * Number of times we encountered bad code.
     * This is a test-only metric.
     */
    constexpr impl::CounterMetric bad_code(33);

    /**
     * generated from test_only.can_we_time_it
     */
    /**
     * Test metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric can_we_time_it(34);

    /**
     * generated from test_only.cheesy_string
     */
    /**
     * Only the cheesiest of strings.
     * This is a test-only metric.
     */
    constexpr impl::StringMetric cheesy_string(35);

    /**
     * generated from test_only.cheesy_string_list
     */
    /**
     * Only the cheesiest of strings. In list form!
     * This is a test-only metric.
     */
    constexpr impl::StringListMetric cheesy_string_list(36);

    /**
     * generated from test_only.what_a_date
     */
    /**
     * ...To be writing FOG code. This is a test-only metric.
     */
    constexpr impl::DatetimeMetric what_a_date(37);

    /**
     * generated from test_only.what_id_it
     */
    /**
     * Just a UUID.
     * This is a test-only metric.
     */
    constexpr impl::UuidMetric what_id_it(38);

    /**
     * generated from test_only.can_we_flag_it
     */
    /**
     * Test metric for a boolean.
     * This is a test-only metric.
     */
    constexpr impl::BooleanMetric can_we_flag_it(39);

    /**
     * generated from test_only.do_you_remember
     */
    /**
     * They say it's the second thing to go.
     * This is a test-only metric.
     */
    constexpr impl::MemoryDistributionMetric do_you_remember(40);

    /**
     * generated from test_only.what_time_is_it
     */
    /**
     * Adheres to at least two of the top ten fallacies programmers believe
     * about time.
     * This is a test-only metric.
     */
    constexpr impl::TimingDistributionMetric what_time_is_it(41);

    /**
     * generated from test_only.mabels_kitchen_counters
     */
    /**
     * Counts Mabels labeled by their kitchen counters.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::CounterMetric> mabels_kitchen_counters(42);

    /**
     * generated from test_only.mabels_bathroom_counters
     */
    /**
     * Counts Mabels labeled by their bathroom counters.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::CounterMetric> mabels_bathroom_counters(43);

    /**
     * generated from test_only.mabels_like_balloons
     */
    /**
     * Does the labeled Mabel like balloons?
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::BooleanMetric> mabels_like_balloons(44);

    /**
     * generated from test_only.mabels_balloon_strings
     */
    /**
     * What do the labeled Mabel's liked balloons' strings say?
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::StringMetric> mabels_balloon_strings(45);

    /**
     * generated from test_only.mabels_label_maker
     */
    /**
     * Mabel just got a label maker and wants to party like it's
     * 1999.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::StringMetric> mabels_label_maker(46);

    /**
     * generated from test_only.mirror_time
     */
    /**
     * Mirrored metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric mirror_time(47);

    /**
     * generated from test_only.mirror_time_nanos
     */
    /**
     * Mirrored metric for a timespan.
     * This is a test-only metric.
     */
    constexpr impl::TimespanMetric mirror_time_nanos(48);

    /**
     * generated from test_only.mirrors_for_labeled_bools
     */
    /**
     * Mirrored metric.
     * This is a test-only metric.
     */
    constexpr impl::Labeled<impl::BooleanMetric> mirrors_for_labeled_bools(49);

    /**
     * generated from test_only.one_ping_one_bool
     */
    /**
     * One bool for one ping only.
     * This is a test-only metric.
     */
    constexpr impl::BooleanMetric one_ping_one_bool(50);

    /**
     * generated from test_only.meaning_of_life
     */
    /**
     * Measures the one true answer to the Ultimate Question of Life,
     * the Universe, and Everything.
     * Approximately.
     * This is a test-only metric.
     */
    constexpr impl::QuantityMetric meaning_of_life(51);

}
namespace test_only_ipc {
    /**
     * generated from test_only.ipc.a_counter
     */
    /**
     * This is a test-only metric.
     * Just counting things.
     */
    constexpr impl::CounterMetric a_counter(52);

    /**
     * generated from test_only.ipc.a_bool
     */
    /**
     * This is a test-only metric.
     * Just flagging things.
     */
    constexpr impl::BooleanMetric a_bool(53);

    /**
     * generated from test_only.ipc.a_date
     */
    /**
     * This is a test-only metric.
     * Just putting things on the calendar.
     */
    constexpr impl::DatetimeMetric a_date(54);

    /**
     * generated from test_only.ipc.a_string
     */
    /**
     * This is a test-only metric.
     * Just setting some strings.
     */
    constexpr impl::StringMetric a_string(55);

    /**
     * generated from test_only.ipc.a_memory_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring memory.
     */
    constexpr impl::MemoryDistributionMetric a_memory_dist(56);

    /**
     * generated from test_only.ipc.a_timing_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring time.
     */
    constexpr impl::TimingDistributionMetric a_timing_dist(57);

    /**
     * generated from test_only.ipc.a_custom_dist
     */
    /**
     * This is a test-only metric.
     * Just measuring samples.
     */
    constexpr impl::CustomDistributionMetric a_custom_dist(58);

    /**
     * generated from test_only.ipc.a_string_list
     */
    /**
     * This is a test-only metric.
     * Just appending some strings.
     */
    constexpr impl::StringListMetric a_string_list(59);

    /**
     * generated from test_only.ipc.an_event
     */
    struct AnEventExtra {
      mozilla::Maybe<nsCString> extra1;
      mozilla::Maybe<nsCString> extra2;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (extra1) {
          extraKeys.AppendElement()->AssignASCII("extra1");
          extraValues.EmplaceBack(extra1.value());
        }
        if (extra2) {
          extraKeys.AppendElement()->AssignASCII("extra2");
          extraValues.EmplaceBack(extra2.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This is a test-only metric.
     * Just recording some events.
     */
    constexpr impl::EventMetric<AnEventExtra> an_event(60);

    /**
     * generated from test_only.ipc.event_with_extra
     */
    struct EventWithExtraExtra {
      mozilla::Maybe<nsCString> extra1;
      mozilla::Maybe<uint32_t> extra2;
      mozilla::Maybe<bool> extra3LongerName;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (extra1) {
          extraKeys.AppendElement()->AssignASCII("extra1");
          extraValues.EmplaceBack(extra1.value());
        }
        if (extra2) {
          extraKeys.AppendElement()->AssignASCII("extra2");
          extraValues.EmplaceBack(nsPrintfCString("%d", extra2.value()));
        }
        if (extra3LongerName) {
          extraKeys.AppendElement()->AssignASCII("extra3_longer_name");
          extraValues.AppendElement()->AssignASCII(extra3LongerName.value() ? "true" : "false");
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This is a test-only metric.
     * Just recording some events.
     */
    constexpr impl::EventMetric<EventWithExtraExtra> event_with_extra(61);

    /**
     * generated from test_only.ipc.no_extra_event
     */
    /**
     * This is a test-only metric.
     * Just recording some events without the extra fuss.
     */
    constexpr impl::EventMetric<NoExtraKeys> no_extra_event(62);

    /**
     * generated from test_only.ipc.a_uuid
     */
    /**
     * This is a test-only metric.
     * Just recording some unique identifiers.
     */
    constexpr impl::UuidMetric a_uuid(63);

    /**
     * generated from test_only.ipc.a_labeled_counter
     */
    /**
     * This is a test-only metric.
     * Just counting labeled things.
     */
    constexpr impl::Labeled<impl::CounterMetric> a_labeled_counter(64);

    /**
     * generated from test_only.ipc.another_labeled_counter
     */
    /**
     * This is a test-only metric.
     * Just another metric counting labeled things.
     */
    constexpr impl::Labeled<impl::CounterMetric> another_labeled_counter(65);

    /**
     * generated from test_only.ipc.a_quantity
     */
    /**
     * This is a test-only metric.
     * Just quantifying things.
     */
    constexpr impl::QuantityMetric a_quantity(66);

    /**
     * generated from test_only.ipc.irate
     */
    /**
     * This is a test-only metric.
     * A rate that isn't happy about it.
     */
    constexpr impl::RateMetric irate(67);

    /**
     * generated from test_only.ipc.rate_with_external_denominator
     */
    /**
     * This is a test-only metric.
     * A rate with a denominator that is Out There.
     */
    constexpr impl::NumeratorMetric rate_with_external_denominator(68);

    /**
     * generated from test_only.ipc.an_external_denominator
     */
    /**
     * This is a test-only metric.
     * A denominator not from around here.
     */
    constexpr impl::DenominatorMetric an_external_denominator(69);

    /**
     * generated from test_only.ipc.a_url
     */
    /**
     * This is a test-only metric.
     * Just setting some Urls.
     */
    constexpr impl::UrlMetric a_url(70);

}
namespace nimbus_events {
    /**
     * generated from nimbus_events.enrollment
     */
    struct EnrollmentExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> enrollmentId;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> experimentType;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (enrollmentId) {
          extraKeys.AppendElement()->AssignASCII("enrollment_id");
          extraValues.EmplaceBack(enrollmentId.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (experimentType) {
          extraKeys.AppendElement()->AssignASCII("experiment_type");
          extraValues.EmplaceBack(experimentType.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a user has met the conditions and is first bucketed into an
     * experiment (i.e. targeting matched and they were randomized into a bucket and
     * branch of the experiment). Expected a maximum of once per experiment per user.
     */
    constexpr impl::EventMetric<EnrollmentExtra> enrollment(71);

    /**
     * generated from nimbus_events.enroll_failed
     */
    struct EnrollFailedExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an enrollment fails, including the reason for the failure.
     */
    constexpr impl::EventMetric<EnrollFailedExtra> enroll_failed(72);

    /**
     * generated from nimbus_events.unenrollment
     */
    struct UnenrollmentExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> enrollmentId;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (enrollmentId) {
          extraKeys.AppendElement()->AssignASCII("enrollment_id");
          extraValues.EmplaceBack(enrollmentId.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when either telemetry is disabled, or the experiment has run for its
     * designed duration (i.e. it is no longer present in the Nimbus Remote Settings
     * collection)
     */
    constexpr impl::EventMetric<UnenrollmentExtra> unenrollment(73);

    /**
     * generated from nimbus_events.unenroll_failed
     */
    struct UnenrollFailedExtra {
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when an unenrollment fails, including the reason for the failure.
     */
    constexpr impl::EventMetric<UnenrollFailedExtra> unenroll_failed(74);

    /**
     * generated from nimbus_events.exposure
     */
    struct ExposureExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> featureId;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (featureId) {
          extraKeys.AppendElement()->AssignASCII("feature_id");
          extraValues.EmplaceBack(featureId.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * Recorded when a user actually observes an experimental treatment, or would have
     * observed an experimental treatment if they had been in a branch that would have
     * shown one.
     */
    constexpr impl::EventMetric<ExposureExtra> exposure(75);

    /**
     * generated from nimbus_events.validation_failed
     */
    struct ValidationFailedExtra {
      mozilla::Maybe<nsCString> branch;
      mozilla::Maybe<nsCString> experiment;
      mozilla::Maybe<nsCString> feature;
      mozilla::Maybe<nsCString> reason;

      Tuple<nsTArray<nsCString>, nsTArray<nsCString>> ToFfiExtra() const {
        nsTArray<nsCString> extraKeys;
        nsTArray<nsCString> extraValues;
        if (branch) {
          extraKeys.AppendElement()->AssignASCII("branch");
          extraValues.EmplaceBack(branch.value());
        }
        if (experiment) {
          extraKeys.AppendElement()->AssignASCII("experiment");
          extraValues.EmplaceBack(experiment.value());
        }
        if (feature) {
          extraKeys.AppendElement()->AssignASCII("feature");
          extraValues.EmplaceBack(feature.value());
        }
        if (reason) {
          extraKeys.AppendElement()->AssignASCII("reason");
          extraValues.EmplaceBack(reason.value());
        }
        return MakeTuple(std::move(extraKeys), std::move(extraValues));
      }
    };
    /**
     * This records when validation of a recipe fails.
     */
    constexpr impl::EventMetric<ValidationFailedExtra> validation_failed(76);

}
namespace power {
    /**
     * generated from power.cpu_time_bogus_values
     */
    /**
     * Impossibly large CPU time values that were discarded.
     */
    constexpr impl::CounterMetric cpu_time_bogus_values(77);

    /**
     * generated from power.cpu_time_per_process_type_ms
     */
    /**
     * CPU time used by each process type in ms.
     */
    constexpr impl::Labeled<impl::CounterMetric> cpu_time_per_process_type_ms(78);

    /**
     * generated from power.gpu_time_per_process_type_ms
     */
    /**
     * GPU time used by each process type in ms.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_time_per_process_type_ms(79);

    /**
     * generated from power.gpu_time_bogus_values
     */
    /**
     * Impossibly large GPU time values that were discarded.
     */
    constexpr impl::CounterMetric gpu_time_bogus_values(80);

    /**
     * generated from power.wakeups_per_process_type
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> wakeups_per_process_type(81);

    /**
     * generated from power.total_cpu_time_ms
     */
    /**
     * Total CPU time used by all processes in ms.
     */
    constexpr impl::CounterMetric total_cpu_time_ms(82);

    /**
     * generated from power.total_gpu_time_ms
     */
    /**
     * Total GPU time used by all processes in ms.
     */
    constexpr impl::CounterMetric total_gpu_time_ms(83);

    /**
     * generated from power.total_thread_wakeups
     */
    /**
     * How many times threads woke up and could have woken up a CPU core.
     */
    constexpr impl::CounterMetric total_thread_wakeups(84);

}
namespace power_wakeups_per_thread {
    /**
     * generated from power.wakeups_per_thread.parent_active
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_active(85);

    /**
     * generated from power.wakeups_per_thread.parent_inactive
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_inactive(86);

    /**
     * generated from power.wakeups_per_thread.content_foreground
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_foreground(87);

    /**
     * generated from power.wakeups_per_thread.content_background
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_background(88);

    /**
     * generated from power.wakeups_per_thread.gpu_process
     */
    /**
     * How many times threads woke up and could have woken up a CPU core. Broken down
     * by thread name for a given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_process(89);

}
namespace power_cpu_ms_per_thread {
    /**
     * generated from power.cpu_ms_per_thread.parent_active
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_active(90);

    /**
     * generated from power.cpu_ms_per_thread.parent_inactive
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> parent_inactive(91);

    /**
     * generated from power.cpu_ms_per_thread.content_foreground
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_foreground(92);

    /**
     * generated from power.cpu_ms_per_thread.content_background
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> content_background(93);

    /**
     * generated from power.cpu_ms_per_thread.gpu_process
     */
    /**
     * How many miliseconds of CPU time were used. Broken down by thread name for a
     * given process type.
     */
    constexpr impl::Labeled<impl::CounterMetric> gpu_process(94);

}
namespace power_battery {
    /**
     * generated from power.battery.percentage_when_user_active
     */
    /**
     * Records how many percent of battery was available for each period of user
     * activity.
     */
    constexpr impl::CustomDistributionMetric percentage_when_user_active(95);

}
namespace search_engine_default {
    /**
     * generated from search.engine.default.engine_id
     */
    /**
     * The telemetry id of the search engine.
     * For application provided engines, this is either supplied by the
     * configuration or from the first part of the associated WebExtension Id.
     * For other engines, this is `other-<extensionName>`.
     */
    constexpr impl::StringMetric engine_id(96);

    /**
     * generated from search.engine.default.display_name
     */
    /**
     * The display name of the user's default engine.
     */
    constexpr impl::StringMetric display_name(97);

    /**
     * generated from search.engine.default.load_path
     */
    /**
     * A path relating to where the search engine was installed/loaded from.
     * For example:
     *   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
     *   engine.
     *   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
     *   engine.
     * Note: this metric is truncated at 100 characters.
     */
    constexpr impl::StringMetric load_path(98);

    /**
     * generated from search.engine.default.submission_url
     */
    /**
     * The submission URL of the default engine. This is only reported in the
     * cases where:
     *   The engine is an application provided engine.
     *   The engine has the same name as an application provided engine.
     *   The engine matches one of a specific list of well known search engines.
     */
    constexpr impl::UrlMetric submission_url(99);

    /**
     * generated from search.engine.default.verified
     */
    /**
     * The verified status of the search engine.
     * For application provided engines, this will always be `default`.
     * For other engines this will either be `verified` or `unverified` depending
     * on if the loadPathHash is valid.
     */
    constexpr impl::StringMetric verified(100);

}
namespace search_engine_private {
    /**
     * generated from search.engine.private.engine_id
     */
    /**
     * The telemetry id of the search engine.
     * For application provided engines, this is either supplied by the
     * configuration or from the first part of the associated WebExtension Id.
     * For other engines, this is `other-<extensionName>`.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric engine_id(101);

    /**
     * generated from search.engine.private.display_name
     */
    /**
     * The display name of the user's default engine.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric display_name(102);

    /**
     * generated from search.engine.private.load_path
     */
    /**
     * A path relating to where the search engine was installed/loaded from.
     * For example:
     *   `[other]addEngineWithDetails:<extension id>` for a WebExtension based
     *   engine.
     *   `[https]developer.mozilla.org/mdn-web-docs.xml` for an OpenSearch based
     *   engine.
     * Note: this metric is truncated at 100 characters.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric load_path(103);

    /**
     * generated from search.engine.private.submission_url
     */
    /**
     * The submission URL of the default engine. This is only reported in the
     * cases where:
     *   The engine is an application provided engine.
     *   The engine has the same name as an application provided engine.
     *   The engine matches one of a specific list of well known search engines.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::UrlMetric submission_url(104);

    /**
     * generated from search.engine.private.verified
     */
    /**
     * The verified status of the search engine.
     * For application provided engines, this will always be `default`.
     * For other engines this will either be `verified` or `unverified` depending
     * on if the loadPathHash is valid.
     * If this string is an empty string (`""`), this means that one or both of
     * the preferences `browser.search.separatePrivateDefault` and
     * `browser.search.separatePrivateDefault.ui.enabled` are set to false.
     * It is possible that the user selects the same private engine as for the
     * default engine, and hence both versions of these fields will be filled in.
     */
    constexpr impl::StringMetric verified(105);

}
namespace legacy_telemetry {
    /**
     * generated from legacy.telemetry.client_id
     */
    /**
     * The client_id according to Telemetry.
     * Might not always have a value due to being too early for it to have
     * loaded.
     * Value may be the canary client id `c0ffeec0-ffee-c0ff-eec0-ffeec0ffeec0`
     * in pings near when the data upload pref is disabled (if Telemetry gets
     * to go first), or between when a client_id has been removed and when it
     * has been regenerated.
     * Does not need to be sent in the Glean "deletion-request" ping.
     */
    constexpr impl::UuidMetric client_id(106);

}
namespace background_update {
    /**
     * generated from background_update.client_id
     */
    /**
     * The Telemetry client ID of the default profile.
     */
    constexpr impl::UuidMetric client_id(107);

    /**
     * generated from background_update.final_state
     */
    /**
     * String description of the final state the update state machine reached.
     */
    constexpr impl::StringMetric final_state(108);

    /**
     * generated from background_update.states
     */
    /**
     * Ordered list of string descriptions of the states that the update state machine
     * reached.
     */
    constexpr impl::StringListMetric states(109);

    /**
     * generated from background_update.reasons
     */
    /**
     * List of reasons that the background update task did not run.
     */
    constexpr impl::StringListMetric reasons(110);

    /**
     * generated from background_update.exit_code_success
     */
    /**
     * True if the exit code/status of the background update task is 0, which means
     * success.
     */
    constexpr impl::BooleanMetric exit_code_success(111);

    /**
     * generated from background_update.exit_code_exception
     */
    /**
     * True if the exit code/status of the background update task is 3, which means an
     * exception was thrown.
     */
    constexpr impl::BooleanMetric exit_code_exception(112);

}
namespace update {
    /**
     * generated from update.service_enabled
     */
    /**
     * Preference "app.update.service.enabled": whether the Mozilla Maintenance
     * Service is enabled.
     */
    constexpr impl::BooleanMetric service_enabled(113);

    /**
     * generated from update.auto_download
     */
    /**
     * Per-installation preference "app.update.auto": whether to fetch and install
     * updates without user intervention.
     */
    constexpr impl::BooleanMetric auto_download(114);

    /**
     * generated from update.background_update
     */
    /**
     * Per-installation preference "app.update.background.enabled": whether to fetch
     * and install updates in the background when Firefox is not running.
     */
    constexpr impl::BooleanMetric background_update(115);

    /**
     * generated from update.enabled
     */
    /**
     * True when policies are disabled or when the "DisableAppUpdate" is not in
     * effect.
     */
    constexpr impl::BooleanMetric enabled(116);

    /**
     * generated from update.channel
     */
    /**
     * The update channel.
     */
    constexpr impl::StringMetric channel(117);

    /**
     * generated from update.can_usually_apply_updates
     */
    /**
     * Whether or not the Update Service can usually download and install updates. See
     * `canUsuallyApplyUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_apply_updates(118);

    /**
     * generated from update.can_usually_check_for_updates
     */
    /**
     * Whether or not the Update Service can usually check for updates. See
     * `canUsuallyCheckForUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_check_for_updates(119);

    /**
     * generated from update.can_usually_stage_updates
     */
    /**
     * Whether the Update Service is usually able to stage updates. See
     * `canUsuallyStageUpdates` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_stage_updates(120);

    /**
     * generated from update.can_usually_use_bits
     */
    /**
     * On Windows, whether the Update Service can usually use BITS. See
     * `canUsuallyUseBits` in https://searchfox.org/mozilla-
     * central/source/toolkit/mozapps/update/nsIUpdateService.idl.
     */
    constexpr impl::BooleanMetric can_usually_use_bits(121);

}
namespace startup {
    /**
     * generated from startup.run_from_dmg_install_outcome
     */
    /**
     * The outcome after the app detected that it was running from DMG or an app
     * translocated location and should offer to install and relaunch itself.
     */
    constexpr impl::Labeled<impl::BooleanMetric> run_from_dmg_install_outcome(122);

}

} // namespace mozilla::glean

#endif // mozilla_Metrics_h
